@@ -1,6 +1,6 @@
 import inspect
 import logging
-from typing import Any, Awaitable, Callable, Literal, Optional
+from typing import Any, Awaitable, Callable, Dict, Literal, Optional
 
 from fastapi.exceptions import HTTPException
 from fastapi.security import OAuth2AuthorizationCodeBearer, SecurityScopes
@@ -22,7 +22,7 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
         app_client_id: str,
         auto_error: bool = True,
         tenant_id: Optional[str] = None,
-        scopes: Optional[dict[str, str]] = None,
+        scopes: Optional[Dict[str, str]] = None,
         multi_tenant: bool = False,
         validate_iss: bool = True,
         iss_callable: Optional[Callable[[str], Awaitable[str]]] = None,
@@ -30,13 +30,14 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
         openid_config_use_app_id: bool = False,
         openapi_authorization_url: Optional[str] = None,
         openapi_token_url: Optional[str] = None,
+        openid_config_url: Optional[str] = None,
         openapi_description: Optional[str] = None,
     ) -> None:
         """
         Initialize settings.
 
         :param app_client_id: str
-            Your applications client ID. This will be the `Web app` in Azure AD
+            Your application client ID. This will be the `Web app` in Azure AD
         :param auto_error: bool
             Whether to throw exceptions or return None on __call__.
         :param tenant_id: str
@@ -70,6 +71,8 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
             Override OpenAPI authorization URL
         :param openapi_token_url: str
             Override OpenAPI token URL
+        :param openid_config_url: str
+            Override OpenID config URL (used for B2C tenants)
         :param openapi_description: str
             Override OpenAPI description
 
@@ -89,7 +92,9 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
             multi_tenant=self.multi_tenant,
             token_version=token_version,
             app_id=app_client_id if openid_config_use_app_id else None,
+            config_url=openid_config_url or None,
         )
+
         self.validate_iss: bool = validate_iss
         self.iss_callable: Optional[Callable[..., Any]] = iss_callable
         self.token_version: int = token_version
@@ -132,17 +137,16 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
                 claims: dict[str, Any] = jwt.get_unverified_claims(token=access_token) or {}
             except Exception as error:
                 log.warning('Malformed token received. %s. Error: %s', access_token, error, exc_info=True)
-                raise InvalidAuth(detail='Invalid token format')
+                raise InvalidAuth(detail='Invalid token format') from error
 
             for scope in security_scopes.scopes:
                 token_scope_string = claims.get('scp', '')
-                if isinstance(token_scope_string, str):
-                    token_scopes = token_scope_string.split(' ')
-                    if scope not in token_scopes:
-                        raise InvalidAuth('Required scope missing')
-                else:
+                if not isinstance(token_scope_string, str):
                     raise InvalidAuth('Token contains invalid formatted scopes')
 
+                token_scopes = token_scope_string.split(' ')
+                if scope not in token_scopes:
+                    raise InvalidAuth('Required scope missing')
             # Load new config if old
             await self.openid_config.load_config()
 
@@ -185,22 +189,22 @@ class AzureAuthorizationCodeBearerBase(SecurityBase):
                         options=options,
                     )
                     # Attach the user to the request. Can be accessed through `request.state.user`
-                    user: User = User(**token | {'claims': token, 'access_token': access_token})
+                    user: User = User(**{**token, 'claims': token, 'access_token': access_token})
                     request.state.user = user
                     return user
             except JWTClaimsError as error:
                 log.info('Token contains invalid claims. %s', error)
-                raise InvalidAuth(detail='Token contains invalid claims')
+                raise InvalidAuth(detail='Token contains invalid claims') from error
             except ExpiredSignatureError as error:
                 log.info('Token signature has expired. %s', error)
-                raise InvalidAuth(detail='Token signature has expired')
+                raise InvalidAuth(detail='Token signature has expired') from error
             except JWTError as error:
                 log.warning('Invalid token. Error: %s', error, exc_info=True)
-                raise InvalidAuth(detail='Unable to validate token')
+                raise InvalidAuth(detail='Unable to validate token') from error
             except Exception as error:
                 # Extra failsafe in case of a bug in a future version of the jwt library
                 log.exception('Unable to process jwt token. Uncaught error: %s', error)
-                raise InvalidAuth(detail='Unable to process token')
+                raise InvalidAuth(detail='Unable to process token') from error
             log.warning('Unable to verify token. No signing keys found')
             raise InvalidAuth(detail='Unable to verify token, no signing keys found')
         except (HTTPException, InvalidAuth):
@@ -215,7 +219,7 @@ class SingleTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase)
         app_client_id: str,
         tenant_id: str,
         auto_error: bool = True,
-        scopes: Optional[dict[str, str]] = None,
+        scopes: Optional[Dict[str, str]] = None,
         token_version: Literal[1, 2] = 2,
         openid_config_use_app_id: bool = False,
         openapi_authorization_url: Optional[str] = None,
@@ -226,7 +230,7 @@ class SingleTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase)
         Initialize settings for a single tenant application.
 
         :param app_client_id: str
-            Your applications client ID. This will be the `Web app` in Azure AD
+            Your application client ID. This will be the `Web app` in Azure AD
         :param tenant_id: str
             Your Azure tenant ID, only needed for single tenant apps
         :param auto_error: bool
@@ -271,7 +275,7 @@ class MultiTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase):
         self,
         app_client_id: str,
         auto_error: bool = True,
-        scopes: Optional[dict[str, str]] = None,
+        scopes: Optional[Dict[str, str]] = None,
         validate_iss: bool = True,
         iss_callable: Optional[Callable[[str], Awaitable[str]]] = None,
         openid_config_use_app_id: bool = False,
@@ -283,7 +287,7 @@ class MultiTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase):
         Initialize settings for a multi-tenant application.
 
         :param app_client_id: str
-            Your applications client ID. This will be the `Web app` in Azure AD
+            Your application client ID. This will be the `Web app` in Azure AD
         :param auto_error: bool
             Whether to throw exceptions or return None on __call__.
         :param scopes: Optional[dict[str, str]
@@ -324,3 +328,63 @@ class MultiTenantAzureAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase):
             openapi_description=openapi_description,
         )
         self.scheme_name: str = 'Azure AD - PKCE, Multi-tenant'
+
+
+class B2CMultiTenantAuthorizationCodeBearer(AzureAuthorizationCodeBearerBase):
+    def __init__(
+        self,
+        app_client_id: str,
+        auto_error: bool = True,
+        scopes: Optional[Dict[str, str]] = None,
+        validate_iss: bool = True,
+        iss_callable: Optional[Callable[[str], Awaitable[str]]] = None,
+        openid_config_use_app_id: bool = False,
+        openid_config_url: Optional[str] = None,
+        openapi_authorization_url: Optional[str] = None,
+        openapi_token_url: Optional[str] = None,
+        openapi_description: Optional[str] = None,
+    ) -> None:
+        """
+        Initialize settings for a B2C multi-tenant application.
+        :param app_client_id: str
+            Your application client ID. This will be the `Web app` in Azure AD
+        :param openid_config_url: str
+            Override OpenID config URL (used for B2C tenants)
+        :param auto_error: bool
+            Whether to throw exceptions or return None on __call__.
+        :param scopes: Optional[dict[str, str]
+            Scopes, these are the ones you've configured in Azure AD. Key is scope, value is a description.
+            Example:
+                {
+                    f'api://{settings.APP_CLIENT_ID}/user_impersonation': 'user impersonation'
+                }
+        :param validate_iss: bool
+            Whether to validate the token `iss` (issuer) or not. This can be skipped to allow anyone to log in.
+        :param iss_callable: Async Callable
+            Async function that has to accept a `tid` (tenant ID) and return a `iss` (issuer) or
+             raise an InvalidIssuer exception
+            This is required when validate_iss is set to `True`.
+        :param openid_config_use_app_id: bool
+            Set this to True if you're using claims-mapping. If you're unsure, leave at False.
+            https://docs.microsoft.com/en-us/azure/active-directory/develop/v2-protocols-oidc#sample-response
+        :param openapi_authorization_url: str
+            Override OpenAPI authorization URL
+        :param openapi_token_url: str
+            Override OpenAPI token URL
+        :param openapi_description: str
+            Override OpenAPI description
+        """
+        super().__init__(
+            app_client_id=app_client_id,
+            auto_error=auto_error,
+            scopes=scopes,
+            validate_iss=validate_iss,
+            iss_callable=iss_callable,
+            multi_tenant=True,
+            openid_config_use_app_id=openid_config_use_app_id,
+            openid_config_url=openid_config_url,
+            openapi_authorization_url=openapi_authorization_url,
+            openapi_token_url=openapi_token_url,
+            openapi_description=openapi_description,
+        )
+        self.scheme_name: str = 'Azure AD - PKCE, B2C Multi-tenant'
